// Convolution class for convolution of two RGB images on the GPU.
//
// Convolution: IFFT { FFT(RGB) * FFT(RGB) }
// Caution: This operation is expensive. It uses nine 2D FFTs in 
// five passes and a number of swizzling passes.
//
// Assumes that GLEW is available and has been initialized.
//
// Two RGB images in OpenGL textures are expected as input.
//
// Code written by Jeppe Revall Frisvad, 2009
// Copyright (c) DTU Informatics 2009

#ifndef CONVOLUTION_H
#define CONVOLUTION_H

#include "FFT.h"

class Convolution
{
public:
  Convolution(unsigned int width, unsigned int height,
              unsigned int in_tex1, unsigned int in_tex2);
  ~Convolution() { delete fft; } 

  void convolve();
  unsigned int get_output() const { return fft_out1_tex; }
  void draw_output() const;

private:
  unsigned int w, h;
  unsigned int input1, input2;
  unsigned int r_prog, g_prog, b_prog;
  unsigned int rg_product_prog, ba_product_prog;
  unsigned int out_prog;
  unsigned int fft_in_tex;
  unsigned int fft_out1_tex;
  unsigned int fft_out2_tex;
  unsigned int fbo;
  FFT* fft;

  void init_texture(unsigned int& tex);
  void init_framebuffer();
  void init_shaders();
  void set_projection() const;
  void unset_projection() const;
  void draw_quad() const;
  void enable_texs() const;
  void disable_texs() const;
};

#endif // CONVOLUTION_H
